/**
 * Project-HomeFlix
 *
 * Copyright 2016-2019  <@Seil0>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 * 
 */

package kellerkinder.HomeFlix.application;

import java.awt.Desktop;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Writer;
import java.math.BigInteger;
import java.net.URLConnection;
import java.time.LocalDate;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.kellerkinder.Alerts.JFXInfoAlert;

import com.eclipsesource.json.Json;
import com.eclipsesource.json.JsonArray;
import com.eclipsesource.json.JsonObject;
import com.jfoenix.controls.JFXButton;
import com.jfoenix.controls.JFXColorPicker;
import com.jfoenix.controls.JFXHamburger;
import com.jfoenix.controls.JFXSlider;
import com.jfoenix.controls.JFXToggleButton;
import com.jfoenix.transitions.hamburger.HamburgerBackArrowBasicTransition;

import javafx.animation.TranslateTransition;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.scene.control.ChoiceBox;
import javafx.scene.control.Label;
import javafx.scene.control.ScrollPane;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;
import javafx.scene.control.TreeItem;
import javafx.scene.effect.BoxBlur;
import javafx.scene.control.ScrollPane.ScrollBarPolicy;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.AnchorPane;
import javafx.scene.layout.FlowPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.scene.text.Font;
import javafx.stage.DirectoryChooser;
import javafx.stage.FileChooser;
import javafx.stage.Stage;
import javafx.util.Duration;
import kellerkinder.HomeFlix.controller.DBController;
import kellerkinder.HomeFlix.controller.OMDbAPIController;
import kellerkinder.HomeFlix.controller.UpdateController;
import kellerkinder.HomeFlix.controller.XMLController;
import kellerkinder.HomeFlix.datatypes.FilmTabelDataType;
import kellerkinder.HomeFlix.datatypes.PosterModeElement;
import kellerkinder.HomeFlix.datatypes.SourceDataType;
import kellerkinder.HomeFlix.player.Player;

public class MainWindowController {	
	
	// general
	@FXML private AnchorPane mainAnchorPane;
	
	@FXML private HBox topHBox;
	@FXML private VBox sideMenuVBox;
	
	@FXML private JFXHamburger menuHam;
	
	@FXML private JFXButton aboutBtn;
	@FXML private JFXButton settingsBtn;

	// settings
	@FXML private ScrollPane settingsScrollPane;
	
	@FXML private JFXButton updateBtn;
	@FXML private JFXButton addDirectoryBtn;
	@FXML private JFXButton addStreamSourceBtn;
	@FXML private JFXToggleButton autoUpdateToggleBtn;
	@FXML private JFXToggleButton autoplayToggleBtn;
	
	@FXML private JFXColorPicker colorPicker;
	
	@FXML private ChoiceBox<String> languageChoisBox = new ChoiceBox<>();
	@FXML private ChoiceBox<String> branchChoisBox = new ChoiceBox<>();

	@FXML private JFXSlider fontsizeSlider;
	
	@FXML private Label homeflixSettingsLbl;
	@FXML private Label mainColorLbl;
	@FXML private Label fontsizeLbl;
	@FXML private Label languageLbl;
	@FXML private Label updateLbl;
	@FXML private Label branchLbl;
	@FXML private Label sourcesLbl;
	@FXML private Label versionLbl;
	
	@FXML private TableView<SourceDataType> sourcesTable;
	@FXML private TreeItem<SourceDataType> sourceRoot = new TreeItem<>(new SourceDataType("", ""));
	@FXML private TableColumn<SourceDataType, String> sourceColumn;
	@FXML private TableColumn<SourceDataType, String> modeColumn;

	// poster-mode
	@FXML private ScrollPane posterModeScrollPane;
	@FXML private FlowPane posterModeFlowPane;

	@FXML private FilmDetailView filmDetailViewController;
	@FXML private SeriesDetailView seriesDetailViewController;
	
	private static MainWindowController instance = null;
	private DBController dbController;
	private UpdateController updateController;
	private XMLController xmlController;
	private Stage primaryStage;
	private static final Logger LOGGER = LogManager.getLogger(MainWindowController.class.getName());

	private boolean menuTrue = false;

	private final String version = "0.7.90";
	private final String buildNumber = "171";
	private final String versionName = "toothless dragon";
	private String btnStyle;

	private FilmTabelDataType currentTableFilm = new FilmTabelDataType("", "", "", "", false, null);

	private ObservableList<String> languages = FXCollections.observableArrayList("English (en_US)", "Deutsch (de_DE)");
	private ObservableList<String> branches = FXCollections.observableArrayList("stable", "beta");
	private ObservableList<FilmTabelDataType> filmsList = FXCollections.observableArrayList();
	private ObservableList<PosterModeElement> posterEmenents = FXCollections.observableArrayList();
	private static ObservableList<SourceDataType> sourcesList = FXCollections.observableArrayList();
	private LocalDate lastValidCache = LocalDate.now().minusDays(30); // current date - 30 days is the last valid cache date

	public MainWindowController() {
		// the constructor
	}
	
	public static MainWindowController getInstance() {
		if (instance == null) {
			LOGGER.error("There was a fatal error: instance is null!");
			instance = new MainWindowController();
		}
		
		return instance;
	}

	@FXML
	public void initialize() {
		instance = this;
		xmlController = new XMLController();
		dbController = DBController.getInstance();
	}

	public void init() {
		LOGGER.info("Initializing Project-HomeFlix build " + buildNumber);

		xmlController.loadSettings(); // load settings
		checkAutoUpdate();

		// initialize the GUI and the DBController
		primaryStage = (Stage) mainAnchorPane.getScene().getWindow(); // set primary stage for dialogs
		initTabel();
		initUI();
		initActions();
		dbController.init();

		// load data list in gui
		addSourceToTable();
		posterModeStartup();
	}

	// Initialize general UI elements
	private void initUI() {
		//JFXScrollPane.smoothScrolling(posterModeScrollPane);
		posterModeScrollPane.setVbarPolicy(ScrollBarPolicy.ALWAYS);
		settingsScrollPane.setVbarPolicy(ScrollBarPolicy.ALWAYS);
		
		versionLbl.setText("Version: " + version + " (Build: " + buildNumber + ")");
		fontsizeSlider.setValue(XMLController.getFontSize());
		colorPicker.setValue(Color.valueOf(XMLController.getColor()));

		updateBtn.setFont(Font.font("System", 12));
		autoUpdateToggleBtn.setSelected(XMLController.isAutoUpdate());
		autoplayToggleBtn.setSelected(XMLController.isAutoplay());
		languageChoisBox.setItems(languages);
		branchChoisBox.setItems(branches);

		if (XMLController.isUseBeta()) {
			branchChoisBox.getSelectionModel().select(1);
		} else {
			branchChoisBox.getSelectionModel().select(0);
		}

		setLocalUI();
		applyColor();
	}

	/**
	 * Initialize the tables (treeTableViewfilm and sourcesTable) only needed for
	 * Tabel-Mode
	 */
	private void initTabel() {
		// sourcesTreeTable
		sourceColumn.setCellValueFactory(cellData -> cellData.getValue().pathProperty());
		modeColumn.setCellValueFactory(cellData -> cellData.getValue().modeProperty());
		sourcesTable.setItems(sourcesList);
	}

	// Initializing the actions
	private void initActions() {

		// general actions
		HamburgerBackArrowBasicTransition burgerTask = new HamburgerBackArrowBasicTransition(menuHam);
		menuHam.addEventHandler(MouseEvent.MOUSE_PRESSED, (e) -> {
			if (menuTrue) {
				sideMenuSlideOut();
				burgerTask.setRate(-1.0);
				burgerTask.play();
				menuTrue = false;
			} else {
				sideMenuSlideIn();
				burgerTask.setRate(1.0);
				burgerTask.play();
				menuTrue = true;
			}
			if (settingsScrollPane.isVisible()) {
				settingsScrollPane.setVisible(false);
			}
		});

		languageChoisBox.getSelectionModel().selectedIndexProperty().addListener((e, oldValue, newValue) -> {
			String local = languageChoisBox.getItems().get((int) newValue).toString();
			local = local.substring(local.length() - 6, local.length() - 1); // reading only en_US from English (en_US)
			XMLController.setUsrLocal(local);
			setLocalUI();
			xmlController.saveSettings();
		});

		branchChoisBox.getSelectionModel().selectedIndexProperty().addListener((e, oldValue, newValue) -> {
			if (branchChoisBox.getItems().get((int) newValue).toString() == "beta") {
				XMLController.setUseBeta(true);
			} else {
				XMLController.setUseBeta(false);
			}
			xmlController.saveSettings();
		});

		fontsizeSlider.valueProperty().addListener(e -> {
			XMLController.setFontSize(fontsizeSlider.getValue());
			
			// TODO add functionality for postermode
			
			xmlController.saveSettings();
		});

		// Poster-Mode actions
	}

	// Table-Mode fxml actions
	@FXML
	private void playbtnclicked() {
		if (currentTableFilm.getStreamUrl().length() > 0) {
			if (currentTableFilm.getStreamUrl().contains("_rootNode")) {
				LOGGER.info("rootNode found, getting last watched episode");
				currentTableFilm = dbController.getLastWatchedEpisode(currentTableFilm.getTitle());
			}

			if (isSupportedFormat(currentTableFilm)) {
				new Player(getCurrentStreamUrl());
			} else {
				LOGGER.error("using fallback player!");
				if (System.getProperty("os.name").contains("Linux")) {
					String line;
					String output = "";
					Process p;
					try {
						p = Runtime.getRuntime().exec("which vlc");
						BufferedReader input = new BufferedReader(new InputStreamReader(p.getInputStream()));
						while ((line = input.readLine()) != null) {
							output = line;
						}
						LOGGER.info("which vlc: " + output);
						input.close();
					} catch (IOException e1) {
						e1.printStackTrace();
					}
					if (output.contains("which: no vlc") || output == "") {
						JFXInfoAlert vlcInfoAlert = new JFXInfoAlert("Info",
								XMLController.getLocalBundle().getString("vlcNotInstalled"), btnStyle, primaryStage);
						vlcInfoAlert.showAndWait();
					} else {
						try {
							new ProcessBuilder("vlc", getCurrentStreamUrl()).start();
						} catch (IOException e) {
							LOGGER.warn("An error has occurred while opening the file!", e);
						}
					}
					
				} else if (System.getProperty("os.name").contains("Windows") || System.getProperty("os.name").contains("Mac OS X")) {
					try {
						Desktop.getDesktop().open(new File(getCurrentStreamUrl()));
					} catch (IOException e) {
						LOGGER.warn("An error has occurred while opening the file!", e);
					}
				} else {
					LOGGER.error(System.getProperty("os.name") + ", OS is not supported, please contact a developer! ");
				}
			}
		}
	}
	
	@FXML
	private void openfolderbtnclicked() {
		File dest = new File(getCurrentStreamUrl()).getParentFile();

		if (!System.getProperty("os.name").contains("Linux")) {
			try {
				Desktop.getDesktop().open(dest);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	// general fxml actions
	@FXML
	private void aboutBtnAction() {
		String bodyText = "Project HomeFlix \nVersion: " + version + " (Build: " + buildNumber + ")  \"" + versionName
				+ "\" \n" + XMLController.getLocalBundle().getString("infoText");
		JFXInfoAlert infoAlert = new JFXInfoAlert("Project HomeFlix", bodyText, btnStyle, primaryStage);
		infoAlert.showAndWait();
	}

	@FXML
	private void settingsBtnclicked() {
		settingsScrollPane.setVisible(!settingsScrollPane.isVisible());
	}

	@FXML
	private void addDirectoryBtnAction() {
		DirectoryChooser directoryChooser = new DirectoryChooser();
		directoryChooser.setTitle(XMLController.getLocalBundle().getString("addDirectory"));
		File selectedFolder = directoryChooser.showDialog(primaryStage);
		if (selectedFolder != null && selectedFolder.exists()) {
			addSource(selectedFolder.getPath(), "local");
		} else {
			LOGGER.error("The selected folder dosen't exist!");
		}
	}

	@FXML
	private void addStreamSourceBtnAction() {
		FileChooser fileChooser = new FileChooser();
		fileChooser.setTitle(XMLController.getLocalBundle().getString("addStreamSource"));
		File selectedFile = fileChooser.showOpenDialog(primaryStage);
		if (selectedFile != null && selectedFile.exists()) {
			addSource(selectedFile.getPath(), "stream");
		} else {
			LOGGER.error("The selected file dosen't exist!");
		}
	}

	@FXML
	private void colorPickerAction() {
		XMLController.setColor(colorPicker.getValue().toString().substring(2, 10));
		xmlController.saveSettings();
		applyColor();
	}

	@FXML
	private void updateBtnAction() {
		updateController = new UpdateController(this, buildNumber, XMLController.isUseBeta());
		Thread updateThread = new Thread(updateController);
		updateThread.setName("Updater");
		updateThread.start();
	}

	@FXML
	private void autoUpdateToggleBtnAction() {
		XMLController.setAutoUpdate(!XMLController.isAutoUpdate());
		xmlController.saveSettings();
	}

	@FXML
	private void autoplayToggleBtnAction() {
		XMLController.setAutoplay(!XMLController.isAutoplay());
		xmlController.saveSettings();
	}

	/**
	 * refresh all films in filmsList and in filmsTable clear the FilmsList and
	 * FilmRoot children, then update the database
	 */
	private void refreshAllFilms() {
		filmsList.clear();
		dbController.refreshDataBase(); // refreshes the database after a source path was added
		filmsList = dbController.getStreamsList(); // returns a list of all films stored in the database
	}

	// add a all elements of sourcesList to the sources table on the settings pane
	public void addSourceToTable() {
		for (SourceDataType source : sourcesList) {
			sourceRoot.getChildren().add(new TreeItem<SourceDataType>(source)); // add data to root-node
		}
	}

	/**
	 * add a source to the newsources list
	 * 
	 * @param path to the source
	 * @param mode of the source (local or streaming)
	 */
	public void addSource(String path, String mode) {
		JsonArray newsources = null;

		try {
			// read old array
			File oldSources = new File(XMLController.getDirHomeFlix() + "/sources.json");
			if (oldSources.exists()) {
				newsources = Json.parse(new FileReader(XMLController.getDirHomeFlix() + "/sources.json")).asArray();
			} else {
				newsources = Json.array();
			}

			// add new source
			JsonObject source = Json.object().add("path", path).add("mode", mode);
			newsources.add(source);
			Writer writer = new FileWriter(XMLController.getDirHomeFlix() + "/sources.json");
			newsources.writeTo(writer);
			writer.close();
		} catch (IOException e) {
			LOGGER.error(e);
		}

		// clear old sources list/table
		getSourcesList().clear();
		sourceRoot.getChildren().clear();

		refreshAllFilms(); // refresh the FilmsList
		checkAllPosters(); // check if there is anything to cache
	}

	/**
	 * set the color of the GUI-Elements if usedColor is less than checkColor set
	 * text fill white, else black
	 */
	private void applyColor() {
		String menuBtnStyle;
		BigInteger usedColor = new BigInteger(XMLController.getColor(), 16);
		BigInteger checkColor = new BigInteger("78909cff", 16);

		if (usedColor.compareTo(checkColor) == -1) {
			btnStyle = "-fx-button-type: RAISED; -fx-background-color: #" + XMLController.getColor() + "; -fx-text-fill: WHITE;";
			menuBtnStyle = "-fx-text-fill: WHITE;";

			menuHam.getStyleClass().clear();
			menuHam.getStyleClass().add("jfx-hamburgerW");
		} else {
			btnStyle = "-fx-button-type: RAISED; -fx-background-color: #" + XMLController.getColor() + "; -fx-text-fill: BLACK;";
			menuBtnStyle = "-fx-text-fill: BLACK;";

			menuHam.getStyleClass().clear();
			menuHam.getStyleClass().add("jfx-hamburgerB");
		}

		// boxes and TextFields
		sideMenuVBox.setStyle("-fx-background-color: #" + XMLController.getColor() + ";");
		topHBox.setStyle("-fx-background-color: #" + XMLController.getColor() + ";");

		// normal buttons
		addDirectoryBtn.setStyle(btnStyle);
		addStreamSourceBtn.setStyle(btnStyle);
		updateBtn.setStyle(btnStyle);

		// menu buttons
		settingsBtn.setStyle(menuBtnStyle);
		aboutBtn.setStyle(menuBtnStyle);
	}

	// slide in in 400ms
	private void sideMenuSlideIn() {
		sideMenuVBox.setVisible(true);
		TranslateTransition translateTransition = new TranslateTransition(Duration.millis(400), sideMenuVBox);
		translateTransition.setFromX(-150);
		translateTransition.setToX(0);
		translateTransition.play();
	}

	// slide out in 400ms
	private void sideMenuSlideOut() {
		TranslateTransition translateTransition = new TranslateTransition(Duration.millis(400), sideMenuVBox);
		translateTransition.setFromX(0);
		translateTransition.setToX(-150);
		translateTransition.play();
	}

	/**
	 * set the local based on the languageChoisBox selection
	 */
	private void setLocalUI() {
		switch (XMLController.getUsrLocal()) {
		case "en_US":
			XMLController.setLocalBundle(ResourceBundle.getBundle("locals.HomeFlix-Local", Locale.US)); // us_English
			languageChoisBox.getSelectionModel().select(0);
			break;
		case "de_DE":
			XMLController.setLocalBundle(ResourceBundle.getBundle("locals.HomeFlix-Local", Locale.GERMAN)); // German
			languageChoisBox.getSelectionModel().select(1);
			break;
		default:
			XMLController.setLocalBundle(ResourceBundle.getBundle("locals.HomeFlix-Local", Locale.US)); // default local
			languageChoisBox.getSelectionModel().select(0);
			break;
		}
		
		filmDetailViewController.updateGUILocal();
		seriesDetailViewController.updateGUILocal();
		
		aboutBtn.setText(XMLController.getLocalBundle().getString("info"));
		settingsBtn.setText(XMLController.getLocalBundle().getString("settings"));
		updateBtn.setText(XMLController.getLocalBundle().getString("checkUpdates"));
		addDirectoryBtn.setText(XMLController.getLocalBundle().getString("addDirectory"));
		addStreamSourceBtn.setText(XMLController.getLocalBundle().getString("addStreamSource"));
		homeflixSettingsLbl.setText(XMLController.getLocalBundle().getString("homeflixSettingsLbl"));
		mainColorLbl.setText(XMLController.getLocalBundle().getString("mainColorLbl"));
		fontsizeLbl.setText(XMLController.getLocalBundle().getString("fontsizeLbl"));
		languageLbl.setText(XMLController.getLocalBundle().getString("languageLbl"));
		autoUpdateToggleBtn.setText(XMLController.getLocalBundle().getString("autoUpdate"));
		autoplayToggleBtn.setText(XMLController.getLocalBundle().getString("autoplay"));
		branchLbl.setText(XMLController.getLocalBundle().getString("branchLbl"));
	}

	// if AutoUpdate, then check for updates
	private void checkAutoUpdate() {

		if (XMLController.isAutoUpdate()) {
			try {
				LOGGER.info("AutoUpdate: looking for updates on startup ...");
				updateController = new UpdateController(this, buildNumber, XMLController.isUseBeta());
				Thread updateThread = new Thread(updateController);
				updateThread.setName("Updater");
				updateThread.start();
				updateThread.join();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * check if a film is supported by the HomeFlixPlayer or not this is the case if
	 * the mime type is mp4
	 * 
	 * @param entry the film you want to check
	 * @return true if so, false if not
	 */
	private boolean isSupportedFormat(FilmTabelDataType film) {
		String mimeType = URLConnection.guessContentTypeFromName(film.getStreamUrl());
		return mimeType != null && (mimeType.contains("mp4") || mimeType.contains("vp6"));
	}

	/**
	 * Poser Mode WIP
	 */

	private void posterModeStartup() {
		checkAllPosters();
		addAllPosters();
		checkCache();
	}

	/**
	 * check if all posters are cached, if not cache the missing ones
	 */
	private void checkAllPosters() {
		// get all not cached entries, none of them should have a cached poster
		ExecutorService executor = Executors.newFixedThreadPool(5);

		for (FilmTabelDataType entry : dbController.getAllNotCachedEntries()) {
			System.out.println(entry.getStreamUrl() + " is NOT cached!");

			Runnable OMDbAPIWorker = new OMDbAPIController(entry, XMLController.getOmdbAPIKey());
			executor.execute(OMDbAPIWorker);
		}
		executor.shutdown();

		// TODO show loading screen
		
		// we might need this as otherwise it would load before all tasks are finished
		try {
			executor.awaitTermination(1, TimeUnit.MINUTES);
		} catch (InterruptedException e) {
			LOGGER.error(e);
		}

		// update all elements from the database
		refreshAllFilms();
		System.out.println("finished refresh");
	}
	
	/**
	 * add all cached films/series to the PosterMode GUI
	 */
	private void addAllPosters() {
		// refresh the posterModeElements list
		posterEmenents.clear();
		posterEmenents = dbController.getPosterElementsList(); // returns a list of all PosterElements stored in the database
		
		// add button onAction
		for (PosterModeElement element : posterEmenents) {
			element.getButton().addEventHandler(MouseEvent.MOUSE_CLICKED, (event) -> {
				enableBlur(); // blur the FlowPane
				System.out.println("selected: " + element.getStreamURL());
				
				// if the selected element is a file it's a film, else a series
				if (new File(element.getStreamURL()).isFile() || element.getStreamURL().contains("http")) {
					filmDetailViewController.setFilm(element.getStreamURL());
					filmDetailViewController.showPane();
				} else {
					seriesDetailViewController.setSeries(element.getStreamURL());
					seriesDetailViewController.showPane();
				}
			});
		}
		
		posterModeFlowPane.getChildren().clear();	// remove all GUIElements from the posterModeFlowPane
		posterModeFlowPane.getChildren().addAll(posterEmenents); // add all films/series as new GUIElements to the posterModeFlowPane
		
		System.out.println("added gui elements");
	}
	
	// TODO can this be done in dbController?
	/**
	 * check if the cache is to old, if so update asynchron
	 */
	private void checkCache() {
		ExecutorService executor = Executors.newFixedThreadPool(5);
		
		// TODO if filmlist is not used anymore, it cann be removed
		for(FilmTabelDataType entry : filmsList) {
			if (dbController.getCacheDate(entry.getStreamUrl()).isBefore(lastValidCache)) {
				System.out.println(entry.getTitle() + " chached on: " + dbController.getCacheDate(entry.getStreamUrl()));
				Runnable OMDbAPIWorker = new OMDbAPIController(entry, XMLController.getOmdbAPIKey());
				executor.execute(OMDbAPIWorker);
			}
		}
		
		executor.shutdown();
	}
	
	private void enableBlur() {
		BoxBlur boxBlur = new BoxBlur();
		boxBlur.setWidth(9);
		boxBlur.setHeight(7);
		boxBlur.setIterations(3);
		posterModeFlowPane.setEffect(boxBlur);
	}
	
	public void disableBlur() {
		posterModeFlowPane.setEffect(null);
	}

	// getter and setter

	public FilmTabelDataType getCurrentTableFilm() {
		return currentTableFilm;
	}

	public String getCurrentTitle() {
		return currentTableFilm.getTitle();
	}

	public String getCurrentStreamUrl() {
		return currentTableFilm.getStreamUrl();
	}

	public static ObservableList<SourceDataType> getSourcesList() {
		return sourcesList;
	}

	public JFXButton getUpdateBtn() {
		return updateBtn;
	}
}
